/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.actions;

import java.io.File;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.WorkspaceImpl;
import com.eviware.soapui.support.SoapUIException;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.support.AbstractSoapUIAction;

/**
 * Action for creating a new Workspace
 * 
 * @author ole.matzura
 */

public class NewWorkspaceAction extends AbstractSoapUIAction<WorkspaceImpl>
{
	public static final String SOAPUI_ACTION_ID = "NewWorkspaceAction";

	public NewWorkspaceAction()
	{
		super( "New Workspace", "Creates a new workspace" );
	}
	
	public void perform( WorkspaceImpl workspace, Object param )
	{
		if( SoapUI.getTestMonitor().hasRunningTests() )
		{
			UISupport.showErrorMessage( "Cannot create and switch workspace white tests are running" );
			return;
		}
		
		String name = UISupport.prompt( "Enter name of new workspace", "New Workspace", "" );
		if( name == null )
			return;
		
		File newPath = UISupport.getFileDialogs().saveAs( 
            this, "New Workspace", ".xml", "soapUI Workspace (*.xml)", 
					new File(name + "-workspace.xml") );
		if( newPath == null )
			return;

		if( SoapUI.getDesktop().closeAll())
		{
			if( newPath.exists() )
			{
				if( !UISupport.confirm( "Workspace exists, overwrite?", "New Workspace" ))
				{
					return;
				}
				
				if( !newPath.delete())
				{
					UISupport.showErrorMessage( "Failed to delete existing workspace" );
					return;
				}
			}
			
			Boolean val = Boolean.TRUE;
			
			if( workspace.getOpenProjectList().size() > 0 )
			{
				val = UISupport.confirmOrCancel( "Save All Projects before Switching Workspace?", "Switch Workspace" );
				if( val == null )
					return;
			}
				
			workspace.save( val.booleanValue() );
			
			try
			{
				workspace.switchWorkspace( newPath );
				SoapUI.getSettings().setString( SoapUI.CURRENT_SOAPUI_WORKSPACE, newPath.getAbsolutePath() );
				workspace.setName( name );
			}
			catch( SoapUIException e )
			{
				UISupport.showErrorMessage( e );
			}
			
		}
	}
}