/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.eviware.soapui.config.LoadTestConfig;
import com.eviware.soapui.config.RunTestCaseStepConfig;
import com.eviware.soapui.config.TestCaseConfig;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.support.XmlBeansPropertiesTestPropertyHolder;
import com.eviware.soapui.impl.wsdl.support.XmlBeansPropertiesTestPropertyHolder.PropertiesStepProperty;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCaseRunner;
import com.eviware.soapui.model.propertyexpansion.PropertyExpansionUtils;
import com.eviware.soapui.model.support.ModelSupport;
import com.eviware.soapui.model.support.TestPropertyListenerAdapter;
import com.eviware.soapui.model.support.TestRunListenerAdapter;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.testsuite.MessageExchangeTestStepResult;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestProperty;
import com.eviware.soapui.model.testsuite.TestPropertyListener;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunListener;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.model.testsuite.TestStepResult.TestStepStatus;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.types.StringList;
import com.eviware.soapui.support.types.StringToObjectMap;

public class WsdlRunTestCaseTestStep extends WsdlTestStep
{
	public static final String TARGET_TESTCASE = WsdlRunTestCaseTestStep.class.getName() + "@target_testcase";
	
	private RunTestCaseStepConfig stepConfig;
	private WsdlTestCaseRunner testCaseRunner;
	private XmlBeansPropertiesTestPropertyHolder propertyHolderSupport;
	private String currentLabel;
	private WsdlTestCase targetTestCase;
	private InternalTestSuiteListener testSuiteListener = new InternalTestSuiteListener();
	private InternalTestRunListener testRunListener = new InternalTestRunListener();
	private InternalTestPropertyListener testPropertyListener = new InternalTestPropertyListener();
	private Set<TestRunListener> testRunListeners = new HashSet<TestRunListener>();

	private WsdlTestCase runningTestCase; 

	public WsdlRunTestCaseTestStep( WsdlTestCase testCase, TestStepConfig config, boolean forLoadTest )
	{
		super( testCase, config, true, forLoadTest );
		
		if( config.getConfig() == null )
		{
			stepConfig = (RunTestCaseStepConfig) config.addNewConfig().changeType( RunTestCaseStepConfig.type );
			stepConfig.addNewProperties();
			stepConfig.addNewReturnProperties();
		}
		else
		{
			stepConfig = (RunTestCaseStepConfig) config.getConfig().changeType( RunTestCaseStepConfig.type );
		}
		
		setIcon( UISupport.createImageIcon( "/run_testcase_step.gif" ) );
		
		propertyHolderSupport = new XmlBeansPropertiesTestPropertyHolder( this, stepConfig.getProperties() );
	}
	
	@Override
	public void afterLoad() 
	{
		setTargetTestCase( findTargetTestCase() );
	}

	private void syncProperties()
	{
		for( String name : propertyHolderSupport.getPropertyNames())
		{
			if( !targetTestCase.hasProperty( name ))
				propertyHolderSupport.removeProperty( name );
		}
		
		for( String name : targetTestCase.getPropertyNames())
		{
			if( !propertyHolderSupport.hasProperty( name ))
				propertyHolderSupport.addProperty( name );
		}
	}

	private WsdlTestCase findTargetTestCase()
	{
		return ModelSupport.findModelItemById( getTestCaseId(), getTestCase().getTestSuite().getProject() );
	}

	public StringList getReturnProperties()
	{
		return new StringList( stepConfig.getReturnProperties().getEntryList() );
	}
	
	public void setReturnProperties( StringList returnProperties )
	{
		stepConfig.getReturnProperties().setEntryArray( returnProperties.toStringArray() );
	}
	
	public TestStepResult run( TestRunner testRunner, TestRunContext testRunContext )
	{
		WsdlMessageExchangeTestStepResult result = new WsdlMessageExchangeTestStepResult( this );
		
		testCaseRunner = null;
		
		if( targetTestCase != null )
		{
			runningTestCase = createTestCase( targetTestCase );
			for( TestRunListener listener : testRunListeners )
				runningTestCase.addTestRunListener( listener );
			
			StringList returnProperties = getReturnProperties();
			Map<String, TestProperty> props = getProperties();
			for( String key : props.keySet() )
			{
				if( runningTestCase.hasProperty( key ) && !returnProperties.contains( key ))
				{
					String value = props.get( key ).getValue();
					runningTestCase.setPropertyValue( key, PropertyExpansionUtils.expandProperties( testRunContext, value ));
				}
			}
			
			currentLabel = getLabel();
			runningTestCase.addTestRunListener( testRunListener );
			
			StringToObjectMap properties = new StringToObjectMap();
			for( String name : testRunContext.getPropertyNames() )
				properties.put( name, testRunContext.getProperty( name ));
			
			result.startTimer();
			testCaseRunner = runningTestCase.run( properties, false );
			result.stopTimer();
			
			for( String key : returnProperties )
			{
				if( runningTestCase.hasProperty( key ))
					setPropertyValue( key, runningTestCase.getPropertyValue( key ) );
			}

			// aggregate results
			for( TestStepResult testStepResult : testCaseRunner.getResults())
			{
				result.addMessages( testStepResult.getMessages() );
				
				if( testStepResult instanceof MessageExchangeTestStepResult )
				{
					result.addMessages( ((MessageExchangeTestStepResult)testStepResult).getMessageExchanges());
				}
			}
			
			switch( testCaseRunner.getStatus() )
			{
				case CANCELED : result.setStatus( TestStepStatus.CANCELED ); break;
				case FAILED : result.setStatus( TestStepStatus.FAILED ); break;
				case FINISHED : result.setStatus( TestStepStatus.OK ); break;
				default : result.setStatus( TestStepStatus.UNKNOWN ); break;
			}
			
			for( TestRunListener listener : testRunListeners )
				runningTestCase.removeTestRunListener( listener );
			
			runningTestCase.release();
			runningTestCase = null;
			testCaseRunner = null;
		}
		else
		{
			result.setStatus( TestStepStatus.FAILED );
			result.addMessage( "Missing testCase in project" );
			result.stopTimer();
		}
		
		return result;
	}
	
	@Override
	public String getLabel()
	{
		String name = getName();
		
		if( testCaseRunner != null )
		{
			name += " - [" + testCaseRunner.getStatus() + "]";
		}
		
		if( isDisabled() )
			return name + " (disabled)";
		else
			return name;
	}

	@Override
	public boolean cancel()
	{
		if( testCaseRunner != null )
		{
			testCaseRunner.cancel( "Canceled by calling TestCase" );
		}
		
		return true;
	}

	private String getTestCaseId()
	{
		return stepConfig.getTargetTestCase();
	}

	public void setTargetTestCase( WsdlTestCase testCase )
	{
		if( targetTestCase != null )
		{
			targetTestCase.getTestSuite().removeTestSuiteListener( testSuiteListener );
			targetTestCase.removeTestPropertyListener( testPropertyListener );
		}
		
		WsdlTestCase oldTestCase = this.targetTestCase;
		this.targetTestCase = testCase;
		
		if( testCase != null )
		{
			stepConfig.setTargetTestCase( testCase.getId() );
			
			targetTestCase.getTestSuite().addTestSuiteListener( testSuiteListener );
			targetTestCase.addTestPropertyListener( testPropertyListener );
			
			syncProperties();
		}
		
		notifyPropertyChanged( TARGET_TESTCASE, oldTestCase, testCase );
	}
	
	/**
	 * Creates a copy of the underlying WsdlTestCase with all LoadTests removed and configured for LoadTesting
	 */
	
	private WsdlTestCase createTestCase( WsdlTestCase testCase )
	{
		// clone config and remove and loadtests
		testCase.beforeSave();
		TestCaseConfig config = (TestCaseConfig) testCase.getConfig().copy();
		config.setLoadTestArray( new LoadTestConfig[0] );
		
		//	 clone entire testCase
		WsdlTestCase wsdlTestCase = new WsdlTestCase( testCase.getTestSuite(), config, true );
		wsdlTestCase.afterLoad();
		return wsdlTestCase;
	}

	public void addTestPropertyListener( TestPropertyListener listener )
	{
		propertyHolderSupport.addTestPropertyListener( listener );
	}

	public Map<String, TestProperty> getProperties()
	{
		return propertyHolderSupport.getProperties();
	}

	public PropertiesStepProperty getProperty( String name )
	{
		return propertyHolderSupport.getProperty( name );
	}

	public String[] getPropertyNames()
	{
		return propertyHolderSupport.getPropertyNames();
	}

	public String getPropertyValue( String name )
	{
		return propertyHolderSupport.getPropertyValue( name );
	}

	public boolean hasProperty( String name )
	{
		return propertyHolderSupport.hasProperty( name );
	}

	public void removeTestPropertyListener( TestPropertyListener listener )
	{
		propertyHolderSupport.removeTestPropertyListener( listener );
	}

	public void setPropertyValue( String name, String value )
	{
		propertyHolderSupport.setPropertyValue( name, value );
	}

	private void updateLabelDuringRun()
	{
		notifyPropertyChanged( WsdlTestStep.LABEL_PROPERTY, currentLabel, getLabel() );
		currentLabel = getLabel();
	}
	
	private final class InternalTestPropertyListener extends TestPropertyListenerAdapter
	{
		@Override
		public void propertyAdded( String name )
		{
			propertyHolderSupport.addProperty( name );
		}

		@Override
		public void propertyRemoved( String name )
		{
			propertyHolderSupport.removeProperty( name );
		}

		@Override
		public void propertyRenamed( String oldName, String newName )
		{
			propertyHolderSupport.renameProperty( oldName, newName );
		}
	}

	private final class InternalTestRunListener extends TestRunListenerAdapter
	{
		@Override
		public void beforeRun( TestRunner testRunner, TestRunContext runContext )
		{
			updateLabelDuringRun();
		}

		@Override
		public void afterRun( TestRunner testRunner, TestRunContext runContext )
		{
			updateLabelDuringRun();
		}

		@Override
		public void afterStep( TestRunner testRunner, TestRunContext runContext, TestStepResult result )
		{
			updateLabelDuringRun();
		}

		@Override
		public void beforeStep( TestRunner testRunner, TestRunContext runContext )
		{
			updateLabelDuringRun();
		}
	}
	
	@Override
	public void resetConfigOnMove( TestStepConfig config )
	{
		super.resetConfigOnMove( config );
		
		stepConfig = (RunTestCaseStepConfig) config.getConfig().changeType(RunTestCaseStepConfig.type);
		propertyHolderSupport.resetPropertiesConfig( stepConfig.getProperties() );
	}

	@Override
	public void release()
	{
		if( targetTestCase != null )
		{
			targetTestCase.getTestSuite().removeTestSuiteListener( testSuiteListener );
			targetTestCase.removeTestPropertyListener( testPropertyListener );
		}
		
		super.release();
	}

	private final class InternalTestSuiteListener extends TestSuiteListenerAdapter
	{
		@Override
		public void testCaseRemoved( TestCase testCase )
		{
			setTargetTestCase( findTargetTestCase() );
		}
	}

	public WsdlTestCase getTargetTestCase()
	{
		return targetTestCase;
	}

	public void addTestRunListener( TestRunListener listener )
	{
		testRunListeners.add( listener );
	}

	public void removeTestRunListener( TestRunListener listener )
	{
		testRunListeners.remove( listener);
	}

	public WsdlTestCase getRunningTestCase()
	{
		return runningTestCase;
	}

	public WsdlTestCaseRunner getTestCaseRunner()
	{
		return testCaseRunner;
	}
}
